import React, {
    useState,
    useEffect,
    useRef,
    createRef,
} from "https://esm.sh/react@18.2.0";
import ReactDOM, { createPortal } from "https://esm.sh/react-dom@18.2.0";
import reactHtml5Parser, {
    parse,
} from "https://cdn.jsdelivr.net/npm/react-html5-parser@0.2.0/+esm";


/* ===============================================================================
 * Utils functions
 * ================================================================================
 */


let articleKeys = ["makeup", "skincare", "perfume"];

const atEmbla = (className) => {
    if (window.innerWidth <= 500) return;

    let emblaNode = document.querySelector(className);
    let viewportNode = emblaNode.querySelector(`.embla__viewport`);

    const options = { loop: true, align: "center", containScroll: false };
    const plugins = [
        EmblaCarouselAutoplay({
            delay: 3000,
            rootNode: (emblaRoot) => emblaRoot.parentElement,
        }),
    ];

    let embla = EmblaCarousel(viewportNode, options, plugins);
};

const imgFadein = () => {
    [].forEach.call(document.querySelectorAll("img[data-src]"), function (img) {
        img.setAttribute("src", img.getAttribute("data-src"));
        img.onload = function () {
            img.removeAttribute("data-src");
        };
    });
};

/* ===============================================================================
 * Component
 * ================================================================================
 */

const modelData = DATA_config;

const Cover = (props) => {
    const { onOpenModel } = props;
    return (
        <div className="grid grid-cols-2 grid-rows-3 w-full h-full p-6">
            <button
                className="icon-btn icon-btn-1 btn self-start place-self-center items-center relative"
                onClick={() => onOpenModel(1)}
            >
                <div className="icon-btn-container relative mx-auto">
                    <img
                        className="h-full"
                        data-src="/anniversary2023/images/theme-icon/icon-1_main.png"
                        alt=""
                    />
                    <div className="animations absolute h-full top-0 left-0 aspect-[0.754]">
                        <div className="eye">
                            <img
                                className="absolute right-[44.3%] top-[23.8%] h-[4%] eye-open"
                                data-src="/anniversary2023/images/theme-icon/icon-1_eye-open.png"
                                alt=""
                            />
                            <img
                                className="absolute right-[44.3%] top-[24%] w-[7%] eye-closed"
                                data-src="/anniversary2023/images/theme-icon/icon-1_eye-closed.png"
                                alt=""
                            />
                        </div>
                        <div className="sparkle">
                            <img
                                className="sparkle-1 absolute left-[13%] top-[10%] w-[14%]"
                                data-src="/anniversary2023/images/theme-icon/icon-1_star-left.png"
                                alt=""
                            />
                            <img
                                className="sparkle-2 absolute right-[13%] top-[30%] w-[19%]"
                                data-src="/anniversary2023/images/theme-icon/icon-1_star-right.png"
                                alt=""
                            />
                        </div>
                    </div>
                </div>
            </button>
            <button
                className="icon-btn icon-btn-2 btn self-start place-self-center relative"
                onClick={() => onOpenModel(2)}
            >
                <div className="icon-btn-container relative mx-auto">
                    <img
                        className="h-full"
                        data-src="/anniversary2023/images/theme-icon/icon-2_main.png"
                        alt=""
                    />
                    <img
                        className="h-full absolute top-0 right-0 left-0 z-10 px-auto  "
                        data-src="/anniversary2023/images/theme-icon/icon-2_main-frame.png"
                        alt=""
                    />
                    <div className="animations absolute h-full top-0 left-0 aspect-[0.754]">
                        <img
                            className="w-[30%] absolute bottom-[28%] right-[32%] hand"
                            data-src="/anniversary2023/images/theme-icon/icon-2_hand.png"
                            alt=""
                        />
                    </div>
                </div>
            </button>

            {/* main  title */}
            <img
                className="max-w-[450px] select-no col-span-2 h-full w-full object-contain mx-auto right-0 left-0 translate-y-[16px]"
                data-src="/anniversary2023/images/main-title.png"
                alt=""
            />
            <button
                className="icon-btn icon-btn-3 btn self-start place-self-center"
                onClick={() => onOpenModel(3)}
            >
                <div className="icon-btn-container relative mx-auto">
                    <img
                        className="h-full  "
                        data-src="/anniversary2023/images/theme-icon/icon-3_main.png"
                        alt=""
                    />
                    <div className="animations absolute h-full top-0 left-0 aspect-[0.754]">
                        <img
                            className="flower-1 w-[30%] absolute top-[25%] left-[-1.2%] -z-10"
                            data-src="/anniversary2023/images/theme-icon/icon-3_flower-1.png"
                            alt=""
                        />
                        <img
                            className="flower-2 w-[13%] absolute bottom-[32%] right-[4%]"
                            data-src="/anniversary2023/images/theme-icon/icon-3_flower-2.png"
                            alt=""
                        />
                        <img
                            className="flower-3 w-[10%] absolute bottom-[43%] right-0"
                            data-src="/anniversary2023/images/theme-icon/icon-3_flower-3.png"
                            alt=""
                        />
                    </div>
                </div>
            </button>
            <button
                className="icon-btn icon-btn-4 btn self-start place-self-center"
                onClick={() => onOpenModel(4)}
            >
                <div className="icon-btn-container relative mx-auto">
                    <img
                        className="h-full  "
                        data-src="/anniversary2023/images/theme-icon/icon-4_main.png"
                        alt=""
                    />
                    <div className="animations absolute h-full top-0 left-0 aspect-[0.754]">
                        <img
                            className="bottle-top w-[18%] absolute top-[10%] right-[13%] "
                            data-src="/anniversary2023/images/theme-icon/icon-4_bottle-top.png"
                            alt=""
                        />
                        <img
                            className="bottle-bottom w-[30%] absolute bottom-[25%] right-[7%] "
                            data-src="/anniversary2023/images/theme-icon/icon-4_bottle-bottom.png"
                            alt=""
                        />
                        <img
                            className="flower w-[14%] absolute bottom-[55%] left-[23%]"
                            data-src="/anniversary2023/images/theme-icon/icon-4_flower.png"
                            alt=""
                        />
                    </div>
                </div>
            </button>
        </div>
    );
};

const Articles = () => {
    useEffect(() => {
        atEmbla(".item-box-1");
        atEmbla(".item-box-2");
        atEmbla(".item-box-3");
    }, []);

    const SingleCh = (props) => {
        const { sortArticles, num } = props;

        let chapter = DATA_config[3].chapter[num - 1];

        return (
            <div className="ch-title flex flex-col items-center gap-2 my-16">
                <img
                    className="w-[100px]"
                    data-src={`/anniversary2023/images/chapter-${num}.png`}
                    alt=""
                />
                <h2 className="title-deco-white">{chapter.title}</h2>
                <div
                    className={`item-box-${num} embla w-full overflow-x-scroll py-3`}
                >
                    <div className="embla__viewport">
                        <div className="embla__container flex md:gap-6 gap-5">
                            {sortArticles?.map((x) => {
                                return (
                                    <a
                                        className="embla__slide"
                                        href={x.link}
                                        target="_blank"
                                    >
                                        <div className="item space-y-1 w-[280px] md:mx-auto ">
                                            <img
                                                className="rounded-xl bg-gray w-[280px] h-[150px] object-cover md:mx-auto"
                                                data-src={x.image}
                                                alt="image"
                                            />
                                            <h4 className="text-white pt-2">
                                                {x.categoryName}
                                            </h4>
                                            <h4 className="item-title">
                                                {x.title}
                                            </h4>
                                        </div>
                                    </a>
                                );
                            })}
                        </div>
                    </div>
                </div>
                <a href={chapter.link} target="_blank">
                    <button className="btn-black">more</button>
                </a>
            </div>
        );
    };

    return (
        <>
            <div className="block-area article-box">
                {articles && (
                    <>
                        {articleKeys.map((x, i) => (
                            <SingleCh num={i + 1} sortArticles={articles[x]} />
                        ))}
                    </>
                )}
            </div>
        </>
    );
};

const DepartmentStoreTimeTable = () => {
    const timeTable = modelData[3].timeTable;
    return (
        <div className="block-area py-10 px-10 flex flex-col gap-6 items-center">
            <h2 className="title-deco-white">全台百貨週慶時間表</h2>
            <div className="area-box space-y-5 w-full">
                {timeTable.time.map((area) => (
                    <div className="area space-y-2  w-full">
                        <div className="black-tag">{area.areaTitle}</div>
                        <table className="w-full">
                            {area.info.map((item) => (
                                <tr className="flex gap-4">
                                    <td>{item.date}</td>
                                    <td>{item.place}</td>
                                </tr>
                            ))}
                        </table>
                    </div>
                ))}
            </div>
            <a href={timeTable.link} target="_blank">
                <button className="btn-black-narrow">看更多百貨時間表</button>
            </a>
        </div>
    );
};

const CmriChamp = () => {
    const productList = modelData[1].products;

    return (
        <div className="block-area bg-white flex flex-col items-center mt-20 mb-10 gap-10 round-top">
            <div className="flex flex-col items-center gap-5">
                <h2 className="title-deco-pink">誰是話題冠軍？！</h2>
                <p>
                    女生每個月都投入了不少時間和金錢在變美這件事上，2023上半年，水水們到底都把錢包貢獻給誰了？大家最關注的彩妝、保養話題是什麼？哪個單品默默成為超受關注的super
                    star？編輯根據「CMRI
                    美妝行銷總研」平台報告，整理了2023上半年臉部保養類及彩妝類關注度比例排名，一起往下看吧～
                </p>
            </div>

            {productList.map((sort) => (
                <div className="sort-box flex flex-col items-center gap-7 mb-4 w-full">
                    <h2 className="item-title font-bold">{sort.areaTitle}</h2>
                    <div className="product-box flex justify-between px-4 w-full items-end">
                        {sort.items.map((item, i) => (
                            <div className="product flex flex-col gap-3 items-center">
                                <div className="relative">
                                    <img
                                        className={item.img_class}
                                        data-src={`/anniversary2023/images/${item.img}.png`}
                                        alt=""
                                    />
                                    <img
                                        className="place-icon animate-pulse absolute right-[-5px] bottom-[-10px] w-8 h-auto"
                                        data-src={`/anniversary2023/images/place_no${
                                            i + 1
                                        }.png`}
                                        alt=""
                                    />
                                </div>
                                <div className="product-info">
                                    <p>{parse(item.name)}</p>
                                    <p>{parse(item.percent)}</p>
                                </div>
                            </div>
                        ))}
                    </div>
                    <a href={sort.link} target="_blank">
                        <button className="btn-black-narrow">
                            看看完整排名及推薦
                        </button>
                    </a>
                </div>
            ))}
        </div>
    );
};

const KolRecommend = () => {
    const kolData = modelData[1].kol;
    return (
        <div className="px-5 py-12 bg-pink flex flex-col gap-5 items-center">
            <h2 className="title-deco-white">原來他們今年就等這一組</h2>

            <div className="kol-box space-y-6">
                {kolData.map((x) => (
                    <div
                        key={x.idx}
                        className="kol-item flex flex-col items-center"
                    >
                        <div className="kol-pic relative flex mb-8 z-10">
                            <img
                                className="h-[200px] w-[200px] rounded-full border-white border-2 bg-gray object-cover"
                                data-src={x.kolImg}
                                alt=""
                            />
                            <div className="absolute w-max bg-pink font-bold rounded-full px-3 py-1 text-black absolute bottom-[-10px] right-0 left-0 mx-auto ">
                                {x.kolName}
                            </div>
                        </div>
                        <div className="white-box flex gap-4 px-6 pb-8">
                            {x.items.map((item) => (
                                <div className="flex-1 recommend-item relative space-y-2">
                                    <div className="top-left-tag absolute left-[-10px] top-[-10px]">
                                        {item.tag}
                                    </div>

                                    <img
                                        className="h-[155px] min-w-[145px] w-full object-cover bg-white"
                                        data-src={item.img}
                                        alt={item.name}
                                    />

                                    <h4 className="item-title">{item.name}</h4>
                                    <p className="text-info">{item.subInfo}</p>
                                    <p className="text-info">{item.info}</p>
                                </div>
                            ))}
                        </div>
                    </div>
                ))}
            </div>
        </div>
    );
};

const Shop = () => {
    const mustBuyTitle = modelData[4].mustBuy;
    const bestOfferTitle = modelData[4].bestOffer;

    const mustBuy = data.products_up;
    const bestOffer = data.products_down;

    const SingleShop = (props) => {
        const { section, sectionTitle, num } = props;

        useEffect(() => {
            atEmbla(`.item-box-${num}`);
        }, []);

        const Stars = (props) => {
            const { rating } = props;
            let ratingArr = [1, 1, 1, 1, 1];

            let noneFillnum = 5 - rating || 0;
            ratingArr.map((x, i, arr) => {
                if (noneFillnum >= i + 1) {
                    arr[i] = 0;
                }
                return ratingArr;
            });

            let newArr = ratingArr
                .sort()
                .reverse()
                .map((x) => (x === 1 ? "fill" : "none"));

            return (
                <div className="flex gap-1">
                    {newArr.map((star) => (
                        <img
                            className="h-4"
                            src={`/anniversary2023/images/rate-star_${star}.png`}
                            alt=""
                        />
                    ))}
                </div>
            );
        };

        return (
            <div className="ch-title flex flex-col items-center gap-2 my-16">
                <h2 className="title-deco-white title-text">
                    {sectionTitle.title}
                </h2>
                <div className={`item-box-${num} embla md:w-full`}>
                    <div className="embla__viewport">
                        <div className="embla__container flex md:gap-6 gap-5 px-5 pt-6 pb-10 md:w-full">
                            {section.map((item) => {
                                return (
                                    <a
                                        className="embla__slide w-[275px] relative mx-auto right-0 left-0"
                                        href={item.link}
                                        target="_blank"
                                    >
                                        <div className="bg-white rounded-2xl w-[275px] h-[460px] overflow-hidden ">
                                            {item.image ? (
                                                <img
                                                    className="bg-white w-[275px] md:w-full h-[235px] object-contain"
                                                    data-src={item.image}
                                                    alt="image"
                                                />
                                            ) : (
                                                <div className="w-[275px] md:w-full h-[235px] bg-white"></div>
                                            )}
                                            <div className="text-box mx-5 my-3">
                                                <h4 className="item-title text-black whitespace-pre-line ellipsis-line-2">
                                                    {parse(item.brand)}
                                                </h4>
                                                <h4 className="item-title text-black whitespace-pre-line ellipsis-line-2">
                                                    {parse(item.title)}
                                                </h4>
                                                <h4 className="item-title text-pink my-2">
                                                    {parse(item.price)}
                                                </h4>
                                                <div className="space-y-2 border-t mt-2 pt-2">
                                                    <div className="rating-box flex items-center gap-3">
                                                        <p className="font-bold">
                                                            必囤指數
                                                        </p>
                                                        <Stars
                                                            rating={parseInt(
                                                                item.score
                                                            )}
                                                        />
                                                    </div>
                                                    <p className="text-small ellipsis-line-3">
                                                        {parse(
                                                            item.description
                                                        )}
                                                    </p>
                                                </div>
                                            </div>
                                        </div>
                                        <button className="btn-black-round absolute bottom-[-20px] right-0 left-0 mx-auto w-max">
                                            我要買呀
                                        </button>
                                    </a>
                                );
                            })}
                        </div>
                    </div>
                </div>
                <a href={sectionTitle.link_moreInfo} target="_blank">
                    <button className="btn-white">
                        more and more 我要囤更多
                    </button>
                </a>
            </div>
        );
    };

    return (
        <div>
            <SingleShop sectionTitle={mustBuyTitle} section={mustBuy} num={1} />
            <SingleShop
                sectionTitle={bestOfferTitle}
                section={bestOffer}
                num={2}
            />
        </div>
    );
};

const Reels = () => {
    const reels = data.reels;

    const lineRefs = useRef([]);
    lineRefs.current = reels.map((_, i) => lineRefs[i] ?? createRef());

    const handlePlayVideo = (vidRef) => {
        if (!!!vidRef) return;
        vidRef.current.play();
    };
    const handlePauseVideo = (vidRef) => {
        if (!!!vidRef) return;
        vidRef.current.pause();
    };

    useEffect(() => {
        atEmbla(".embla");
    }, []);

    // 手機版會自動播放所有影片
    useEffect(() => {
        if (window.innerWidth <= 500) {
            lineRefs.current.forEach((x, i) => {
                return handlePlayVideo(x);
            });
        }
    }, []);

    return (
        <div className="flex flex-col items-center my-4">
            <div className="embla md:w-full">
                <div className="embla__viewport">
                    <div className="embla__container flex gap-5 item-box md:w-full px-5 py-10 my-2">
                        {reels.map((reel, i) => (
                            <a
                                className="embla__slide"
                                href={reel.link}
                                target="_blank"
                            >
                                <div
                                    className="space-y-2 w-[312px] md:mx-auto"
                                    onMouseEnter={() => {
                                        if (window.innerWidth <= 500) return;
                                        handlePlayVideo(lineRefs.current[i]);
                                    }}
                                    onMouseLeave={() => {
                                        if (window.innerWidth <= 500) return;
                                        handlePauseVideo(lineRefs.current[i]);
                                    }}
                                >
                                    <video
                                        ref={lineRefs.current[i]}
                                        playsinline="true"
                                        webkit-playsinline="true"
                                        loop
                                        muted
                                        preload="metadata"
                                        className="bg-white w-[312px] h-[560px] object-cover"
                                    >
                                        <source
                                            src={`${reel.video_url}#t=0.001`}
                                            type="video/mp4"
                                            alt="reel"
                                        />
                                        Your browser does not support the video
                                        tag.
                                    </video>
                                    <h2 className="whitespace-pre-warp text-white ellipsis-line-1">
                                        {parse(reel.title)}
                                    </h2>
                                    <p className="text-small ellipsis-line-3">
                                        {parse(reel.description)}
                                    </p>
                                </div>
                            </a>
                        ))}
                    </div>
                </div>
            </div>
            <a
                className="more"
                href="https://www.instagram.com/girlstalk.tw/reels/"
                target="_blank"
            >
                <button className="btn-black">more</button>
            </a>
        </div>
    );
};

const Footer = () => {
    const footerIcons = DATA_config.footer;

    return (
        <footer>
            <div className="w-full flex flex-col gap-10 items-center my-12">
                <div className="logo-item flex flex-col gap-4 items-center">
                    <p className="font-bold tracking-[2.8px] text-center">
                        主辦單位
                    </p>
                    <img
                        data-src="/anniversary2023/images/logo-girlstalk.png"
                        alt=""
                    />
                </div>
                <div className="logo-item flex flex-col gap-4 items-center">
                    <p className="font-bold tracking-[2.8px] text-center">
                        合作單位
                    </p>
                    <img
                        className="mb-6"
                        data-src="/anniversary2023/images/logo-elemis_london.png"
                        alt=""
                    />
                    <img data-src="/anniversary2023/images/logo-1.png" alt="" />
                </div>
            </div>
            <div className="w-full bg-black text-white py-7 px-6">
                <div className="flex flex-col mx-6 items-center">
                    <div className="flex gap-8 items-center">
                        <p className="tracking-[2.8px] text-center">
                            追蹤最新消息
                        </p>
                        <div className="flex gap-6">
                            {footerIcons &&
                                footerIcons.map((icon, key) => (
                                    <a
                                        key={key}
                                        href={icon.link}
                                        target="_blank"
                                    >
                                        <img
                                            className="h-8 md:h-12"
                                            data-src={icon.icon}
                                            alt="icon"
                                        />
                                    </a>
                                ))}
                        </div>
                    </div>
                    <div className="mt-3">
                        <p className="font-light">
                            2023 GIRLSTALK © All Rights Reserved.
                        </p>
                    </div>
                </div>
            </div>
        </footer>
    );
};

/* ===============================================================================
 * Model
 * ================================================================================
 */

const Model = (props) => {
    const { onOpenModel, openModel } = props;
    useEffect(() => {
        imgFadein();
    }, []);
    //  * Component order
    const BLOCK_ORDER = {
        1: [<CmriChamp />, <KolRecommend />],
        2: [<Reels />],
        3: [<Articles />, <DepartmentStoreTimeTable />],
        4: [<Shop />],
    };

    const BackBtn = () => {
        return (
            <button
                className="btn fixed top-0 p-6 z-30"
                onClick={() => onOpenModel(0)}
            >
                <img
                    className="h-14 w-14"
                    src="/anniversary2023/images/icon_back.svg"
                    alt=""
                />
            </button>
        );
    };
    return (
        <div className="w-full bg-pink absolute h-full top-0 z-20 ">
            <div className="overflow-y-scroll overflow-x-hidden h-full w-full relative">
                <BackBtn />
                <div className="area-container mt-24">
                    <div className="block-area title-area">
                        <div className="relative">
                            <img
                                className="h-[100px]"
                                src={`/anniversary2023/images/theme-title-${openModel}.png`}
                                alt=""
                            />
                            <img
                                className="absolute top-0 right-0 w-[130px]"
                                src={`/anniversary2023/images/theme-subtitle-${openModel}.png`}
                                alt=""
                            />
                            {openModel === 2 && (
                                <img
                                    className="h-7 absolute right-0 bottom-0"
                                    data-src="/anniversary2023/images/small-title.png"
                                    alt=""
                                />
                            )}
                        </div>

                        <p className="mt-4">{modelData[openModel].intro}</p>
                    </div>
                    {BLOCK_ORDER[openModel]}
                </div>
                <Footer />
            </div>
        </div>
    );
};

/* ===============================================================================
 * App entrypoint
 * ================================================================================
 */

const App = () => {
    const [openModel, setOpenModel] = useState(0);
    // 0 -> 關閉model | 1-4 開啟1-4個別頁面

    useEffect(() => {
        imgFadein();
    }, []);

    return (
        <div className="bg-wrapper w-full h-full flex flex-col gap-10 items-center fixed top-0">
            <div className="lg-bg-deco fixed top-0">
                <img
                    className="fixed top-6 left-16 h-[63%]"
                    data-src="/anniversary2023/images/bg-deco-1.png"
                    alt="bg-deco"
                />
                <img
                    className="fixed bottom-6 left-16 h-[10%]"
                    data-src="/anniversary2023/images/bg-deco-2.png"
                    alt="bg-deco"
                />
                <img
                    className="fixed bottom-6 right-10 h-[65%]"
                    data-src="/anniversary2023/images/bg-deco-3.png"
                    alt="bg-deco"
                />
            </div>
            <div className="mobile-wrapper relative h-full w-full max-w-[500px] ">
                {/* min-w-[375px] */}
                {openModel !== 0 && (
                    <Model onOpenModel={setOpenModel} openModel={openModel} />
                )}
                <Cover onOpenModel={setOpenModel} />
            </div>
        </div>
    );
};
export default App;

ReactDOM.render(<App />, document.querySelector("#root"));
